@@ -112,8 +112,10 @@ submodules = [atmosphere, compressible, core, friction, filters, fittings,
               saltation, separator, particle_size_distribution, jet_pump,
               control_valve]
 
-__version__ = '0.1.80'
-
-fluids_dir = os.path.dirname(__file__)
-fluids_data_dir = os.path.join(fluids_dir, 'data')
+__version__ = '0.1.82'
 
+try:
+    fluids_dir = os.path.dirname(__file__)
+    fluids_data_dir = os.path.join(fluids_dir, 'data')
+except:
+    pass
