@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''Chemical Engineering Design Library (ChEDL). Utilities for process modeling.
-Copyright (C) 2016, Caleb Bell <Caleb.Andrew.Bell@gmail.com>
+Copyright (C) 2016, 2017, 2018, 2019, 2020 Caleb Bell <Caleb.Andrew.Bell@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -18,23 +18,67 @@ FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-SOFTWARE.'''
-
+SOFTWARE.
+'''
 from __future__ import division
-from math import sin, exp, pi
-from scipy.constants import g, R
-import numpy as np
+'''
+Additional copyright:
+The functions C2K, K2C, F2C, C2F, F2K, K2F, C2R, K2R, F2R, R2C, R2K, R2F
+were deprecated from scipy but are still wanted by fluids
+Taken from scipy/constants/constants.py as in commit 
+https://github.com/scipy/scipy/commit/4b7d325cd50e8828b06d628e69426a18283dc5b5
+Also from https://github.com/scipy/scipy/pull/5292
+by Gillu13  (Gilles Aouizerate)
+They are copyright individual contributors to SciPy, under the BSD 3-Clause
+The license of scipy is as follows:
+
+    Copyright (c) 2001-2002 Enthought, Inc.  2003-2019, SciPy Developers.
+All rights reserved.
+
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions
+are met:
+
+1. Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+
+2. Redistributions in binary form must reproduce the above
+   copyright notice, this list of conditions and the following
+   disclaimer in the documentation and/or other materials provided
+   with the distribution.
+
+3. Neither the name of the copyright holder nor the names of its
+   contributors may be used to endorse or promote products derived
+   from this software without specific prior written permission.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+'''
+
+from math import sin, exp, pi, fabs, copysign
+from fluids.constants import g, R
 
 __all__ = ['Reynolds', 'Prandtl', 'Grashof', 'Nusselt', 'Sherwood', 'Rayleigh',
 'Schmidt', 'Peclet_heat', 'Peclet_mass', 'Fourier_heat', 'Fourier_mass',
-'Graetz_heat', 'Lewis', 'Weber', 'Mach', 'Knudsen', 'Bond', 'Dean',
+'Graetz_heat', 'Lewis', 'Weber', 'Mach', 'Knudsen', 'Bond', 'Dean', 'Morton',
 'Froude', 'Froude_densimetric', 'Strouhal', 'Biot', 'Stanton', 'Euler', 'Cavitation', 'Eckert',
 'Jakob', 'Power_number', 'Stokes_number', 'Drag', 'Capillary', 'Bejan_L', 'Bejan_p', 'Boiling',
 'Confinement', 'Archimedes', 'Ohnesorge', 'Suratman', 'Hagen', 'thermal_diffusivity', 'c_ideal_gas',
 'relative_roughness', 'nu_mu_converter', 'gravity',
 'K_from_f', 'K_from_L_equiv', 'L_equiv_from_K', 'L_from_K', 'dP_from_K', 
 'head_from_K', 'head_from_P',
-'P_from_head', 'Eotvos']
+'P_from_head', 'Eotvos',
+'C2K', 'K2C', 'F2C', 'C2F', 'F2K', 'K2F', 'C2R', 'K2R', 'F2R', 'R2C', 'R2K', 'R2F',
+]
 
 
 ### Not quite dimensionless groups
@@ -109,7 +153,7 @@ def c_ideal_gas(T, k, MW):
     Examples
     --------
     >>> c_ideal_gas(T=303, k=1.4, MW=28.96)
-    348.9820361755092
+    348.9820953185441
 
     References
     ----------
@@ -175,10 +219,10 @@ def Reynolds(V, D, rho=None, mu=None, nu=None):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    if rho and mu:
+    if rho is not None and mu is not None:
         nu = mu/rho
-    elif not nu:
-        raise Exception('Either density and viscosity, or dynamic viscosity, \
+    elif nu is None:
+        raise ValueError('Either density and viscosity, or dynamic viscosity, \
         is needed')
     return V*D/nu
 
@@ -237,9 +281,9 @@ def Peclet_heat(V, L, rho=None, Cp=None, k=None, alpha=None):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    if rho and Cp and k:
+    if rho is not None and Cp is not None and k is not None:
         alpha =  k/(rho*Cp)
-    elif not alpha:
+    elif alpha is None:
         raise Exception('Either heat capacity and thermal conductivity and\
         density, or thermal diffusivity is needed')
     return V*L/alpha
@@ -339,12 +383,12 @@ def Fourier_heat(t, L, rho=None, Cp=None, k=None, alpha=None):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    if rho and Cp and k:
+    if rho is not None and Cp is not None and k is not None:
         alpha =  k/(rho*Cp)
-    elif not alpha:
+    elif alpha is None:
         raise Exception('Either heat capacity and thermal conductivity and \
 density, or thermal diffusivity is needed')
-    return t*alpha/L**2
+    return t*alpha/(L*L)
 
 
 def Fourier_mass(t, L, D):
@@ -383,7 +427,7 @@ def Fourier_mass(t, L, D):
     .. [1] Green, Don, and Robert Perry. Perry's Chemical Engineers' Handbook,
        Eighth Edition. McGraw-Hill Professional, 2007.
     '''
-    return t*D/L**2
+    return t*D/(L*L)
 
 
 def Graetz_heat(V, D, x, rho=None, Cp=None, k=None, alpha=None):
@@ -427,6 +471,7 @@ def Graetz_heat(V, D, x, rho=None, Cp=None, k=None, alpha=None):
         Gz = \frac{\text{Time for radial heat diffusion in a fluid by conduction}}
         {\text{Time taken by fluid to reach distance x}}
 
+    .. math::
         Gz = \frac{D}{x}RePr
 
     An error is raised if none of the required input sets are provided.
@@ -444,12 +489,12 @@ def Graetz_heat(V, D, x, rho=None, Cp=None, k=None, alpha=None):
        David P. DeWitt. Introduction to Heat Transfer. 6E. Hoboken, NJ:
        Wiley, 2011.
     '''
-    if rho and Cp and k:
-        alpha =  k/(rho*Cp)
-    elif not alpha:
+    if rho is not None and Cp is not None and k is not None:
+        alpha = k/(rho*Cp)
+    elif alpha is None:
         raise Exception('Either heat capacity and thermal conductivity and\
         density, or thermal diffusivity is needed')
-    return V*D**2/(x*alpha)
+    return V*D*D/(x*alpha)
 
 
 def Schmidt(D, mu=None, nu=None, rho=None):
@@ -502,12 +547,12 @@ def Schmidt(D, mu=None, nu=None, rho=None):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    if rho and mu:
+    if rho is not None and mu is not None:
         return mu/(rho*D)
-    elif nu:
+    elif nu is not None:
         return nu/D
     else:
-        raise Exception('Insufficient information provided for Schmidt number calculation')
+        raise ValueError('Insufficient information provided for Schmidt number calculation')
 
 
 def Lewis(D=None, alpha=None, Cp=None, k=None, rho=None):
@@ -563,11 +608,9 @@ def Lewis(D=None, alpha=None, Cp=None, k=None, rho=None):
     .. [3] Gesellschaft, V. D. I., ed. VDI Heat Atlas. 2nd edition.
        Berlin; New York:: Springer, 2010.
     '''
-    if k and Cp and rho:
+    if k is not None and Cp is not None and rho is not None:
         alpha = k/(rho*Cp)
-    elif alpha:
-        pass
-    else:
+    elif alpha is None:
         raise Exception('Insufficient information provided for Le calculation')
     return alpha/D
 
@@ -617,7 +660,7 @@ def Weber(V, L, rho, sigma):
     .. [3] Gesellschaft, V. D. I., ed. VDI Heat Atlas. 2nd edition.
        Berlin; New York:: Springer, 2010.
     '''
-    return V**2*L*rho/sigma
+    return V*V*L*rho/sigma
 
 
 def Mach(V, c):
@@ -718,6 +761,53 @@ def Confinement(D, rhol, rhog, sigma, g=g):
     return (sigma/(g*(rhol-rhog)))**0.5/D
 
 
+def Morton(rhol, rhog, mul, sigma, g=g):
+    r'''Calculates Morton number or `Mo` for a liquid and vapor with the
+    specified properties, under the influence of gravitational force `g`.
+
+    .. math::
+        Mo = \frac{g \mu_l^4(\rho_l - \rho_g)}{\rho_l^2 \sigma^3}
+        
+    Parameters
+    ----------
+    rhol : float
+        Density of liquid phase, [kg/m^3]
+    rhog : float
+        Density of gas phase, [kg/m^3]
+    mul : float
+        Viscosity of liquid phase, [Pa*s]
+    sigma : float
+        Surface tension between liquid-gas phase, [N/m]
+    g : float, optional
+        Acceleration due to gravity, [m/s^2]
+
+    Returns
+    -------
+    Mo : float
+        Morton number, [-]
+
+    Notes
+    -----
+    Used in modeling bubbles in liquid.
+
+    Examples
+    --------
+    >>> Morton(1077.0, 76.5, 4.27E-3, 0.023)
+    2.311183104430743e-07
+
+    References
+    ----------
+    .. [1] Kunes, Josef. Dimensionless Physical Quantities in Science and 
+       Engineering. Elsevier, 2012.
+    .. [2] Yan, Xiaokang, Kaixin Zheng, Yan Jia, Zhenyong Miao, Lijun Wang,
+       Yijun Cao, and Jiongtian Liu. “Drag Coefficient Prediction of a Single 
+       Bubble Rising in Liquids.” Industrial & Engineering Chemistry Research, 
+       April 2, 2018. https://doi.org/10.1021/acs.iecr.7b04743.
+    '''
+    mul2 = mul*mul
+    return g*mul2*mul2*(rhol - rhog)/(rhol*rhol*sigma*sigma*sigma)
+
+
 def Knudsen(path, L):
     r'''Calculates Knudsen number or `Kn` for a fluid with mean free path
     `path` and for a characteristic length `L`.
@@ -817,14 +907,14 @@ def Prandtl(Cp=None, k=None, mu=None, nu=None, rho=None, alpha=None):
     .. [3] Gesellschaft, V. D. I., ed. VDI Heat Atlas. 2nd edition.
        Berlin; New York:: Springer, 2010.
     '''
-    if k and Cp and mu:
+    if k is not None and Cp is not None and mu is not None:
         return Cp*mu/k
-    elif nu and rho and Cp and k:
+    elif nu is not None and rho is not None and Cp is not None and k is not None:
         return nu*rho*Cp/k
-    elif nu and alpha:
+    elif nu is not None and alpha is not None:
         return nu/alpha
     else:
-        raise Exception('Insufficient information provided for Pr calculation')
+        raise ValueError('Insufficient information provided for Pr calculation')
 
 
 def Grashof(L, beta, T1, T2=0, rho=None, mu=None, nu=None, g=g):
@@ -889,12 +979,12 @@ def Grashof(L, beta, T1, T2=0, rho=None, mu=None, nu=None, g=g):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    if rho and mu:
+    if rho is not None and mu is not None:
         nu = mu/rho
-    elif not nu:
-        raise Exception('Either density and viscosity, or dynamic viscosity, \
+    elif nu is None:
+        raise ValueError('Either density and viscosity, or dynamic viscosity, \
         is needed')
-    return g*beta*abs(T2-T1)*L**3/nu**2
+    return g*beta*abs(T2-T1)*L*L*L/(nu*nu)
 
 
 def Bond(rhol, rhog, sigma, L):
@@ -931,7 +1021,7 @@ def Bond(rhol, rhog, sigma, L):
     .. [1] Green, Don, and Robert Perry. Perry's Chemical Engineers' Handbook,
        Eighth Edition. McGraw-Hill Professional, 2007.
     '''
-    return (g*(rhol-rhog)*L**2/sigma)
+    return (g*(rhol-rhog)*L*L/sigma)
 
 Eotvos = Bond
 
@@ -1013,7 +1103,7 @@ def Froude(V, L, g=g, squared=False):
     >>> Froude(1.83, L=2., g=1.63)
     1.0135432593877318
     >>> Froude(1.83, L=2., squared=True)
-    0.17074638128208924
+    0.17074638128208922
 
     References
     ----------
@@ -1022,7 +1112,7 @@ def Froude(V, L, g=g, squared=False):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    Fr = V/(L*g)**0.5
+    Fr = V*(L*g)**-0.5
     if squared:
         Fr *= Fr
     return Fr
@@ -1364,7 +1454,7 @@ def Euler(dP, rho, V):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    return dP/(rho*V**2)
+    return dP/(rho*V*V)
 
 
 def Cavitation(P, Psat, rho, V):
@@ -1411,7 +1501,7 @@ def Cavitation(P, Psat, rho, V):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    return (P-Psat)/(0.5*rho*V**2)
+    return (P-Psat)/(0.5*rho*V*V)
 
 
 def Eckert(V, Cp, dT):
@@ -1452,7 +1542,7 @@ def Eckert(V, Cp, dT):
     .. [1] Goldstein, Richard J. ECKERT NUMBER. Thermopedia. Hemisphere, 2011.
        10.1615/AtoZ.e.eckert_number
     '''
-    return V**2/(Cp*dT)
+    return V*V/(Cp*dT)
 
 
 def Jakob(Cp, Hvap, Te):
@@ -1543,7 +1633,7 @@ def Power_number(P, L, N, rho):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    return P/(rho*N**3*L**5)
+    return P/(rho*N*N*N*L**5)
 
 
 def Drag(F, A, V, rho):
@@ -1589,7 +1679,7 @@ def Drag(F, A, V, rho):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    return F/(A*rho*V**2/2.)
+    return F/(0.5*A*rho*V*V)
 
 
 def Stokes_number(V, Dp, D, rhop, mu):
@@ -1729,7 +1819,7 @@ def Archimedes(L, rhof, rhop, mu, g=g):
     .. [2] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    return L**3*rhof*(rhop-rhof)*g/mu**2
+    return L*L*L*rhof*(rhop-rhof)*g/(mu*mu)
 
 
 def Ohnesorge(L, rho, mu, sigma):
@@ -1766,14 +1856,14 @@ def Ohnesorge(L, rho, mu, sigma):
     Examples
     --------
     >>> Ohnesorge(1E-4, 1000., 1E-3, 1E-1)
-    0.01
+    0.009999999999999998
 
     References
     ----------
     .. [1] Green, Don, and Robert Perry. Perry's Chemical Engineers' Handbook,
        Eighth Edition. McGraw-Hill Professional, 2007.
     '''
-    return mu/(L*rho*sigma)**0.5
+    return mu*(L*rho*sigma)**-0.5
 
     
 def Suratman(L, rho, mu, sigma):
@@ -1924,7 +2014,7 @@ def Bejan_L(dP, L, mu, alpha):
     .. [2] Bejan, Adrian. Convection Heat Transfer. 4E. Hoboken, New Jersey:
        Wiley, 2013.
     '''
-    return dP*L**2/(alpha*mu)
+    return dP*L*L/(alpha*mu)
 
 
 def Bejan_p(dP, K, mu, alpha):
@@ -2115,6 +2205,7 @@ def nu_mu_converter(rho, mu=None, nu=None):
     .. math::
         \nu = \frac{\mu}{\rho}
 
+    .. math::
         \mu = \nu\rho
 
     Parameters
@@ -2141,11 +2232,11 @@ def nu_mu_converter(rho, mu=None, nu=None):
     .. [1] Cengel, Yunus, and John Cimbala. Fluid Mechanics: Fundamentals and
        Applications. Boston: McGraw Hill Higher Education, 2006.
     '''
-    if (nu and mu) or not rho or (not nu and not mu):
-        raise Exception('Inputs must be rho and one of mu and nu.')
-    if mu:
+    if (nu is not None and mu is not None) or rho is None or (nu is None and mu is None):
+        raise ValueError('Inputs must be rho and one of mu and nu.')
+    if mu is not None:
         return mu/rho
-    elif nu:
+    else:
         return nu*rho
 
 
@@ -2454,16 +2545,9 @@ alpha = thermal_diffusivity # synonym for thermal diffusivity
 Pr = Prandtl # Synonym
 
 
-# The following are functions which were deprecated from scipy
-# but are still desired to be here
-# Taken from scipy/constants/constants.py as in commit 
-# https://github.com/scipy/scipy/commit/4b7d325cd50e8828b06d628e69426a18283dc5b5
-# Copyright individual contributors to SciPy
-
-
 # temperature in kelvin
 zero_Celsius = 273.15
-degree_Fahrenheit = 1/1.8 # only for differences
+degree_Fahrenheit = 1.0/1.8 # only for differences
 
 def C2K(C):
     """
@@ -2471,13 +2555,13 @@ def C2K(C):
 
     Parameters
     ----------
-    C : array_like
-        Celsius temperature(s) to be converted.
+    C : float
+        Celsius temperature to be converted, [degC]
 
     Returns
     -------
-    K : float or array of floats
-        Equivalent Kelvin temperature(s).
+    K : float
+        Equivalent Kelvin temperature, [K]
 
     Notes
     -----
@@ -2486,12 +2570,10 @@ def C2K(C):
 
     Examples
     --------
-    >>> from fluids.core import C2K
-    >>> C2K(np.array([-40, 40.0]))
-    array([ 233.15,  313.15])
-
+    >>> C2K(-40)
+    233.14999999999998
     """
-    return np.asanyarray(C) + zero_Celsius
+    return C + zero_Celsius
 
 
 def K2C(K):
@@ -2500,13 +2582,13 @@ def K2C(K):
 
     Parameters
     ----------
-    K : array_like
-        Kelvin temperature(s) to be converted.
+    K : float
+        Kelvin temperature to be converted.
 
     Returns
     -------
-    C : float or array of floats
-        Equivalent Celsius temperature(s).
+    C : float
+        Equivalent Celsius temperature.
 
     Notes
     -----
@@ -2515,12 +2597,10 @@ def K2C(K):
 
     Examples
     --------
-    >>> from fluids.core import K2C
-    >>> K2C(np.array([233.15, 313.15]))
-    array([-40.,  40.])
-
+    >>> K2C(233.15)
+    -39.99999999999997
     """
-    return np.asanyarray(K) - zero_Celsius
+    return K - zero_Celsius
 
 
 def F2C(F):
@@ -2529,13 +2609,13 @@ def F2C(F):
 
     Parameters
     ----------
-    F : array_like
-        Fahrenheit temperature(s) to be converted.
+    F : float
+        Fahrenheit temperature to be converted.
 
     Returns
     -------
-    C : float or array of floats
-        Equivalent Celsius temperature(s).
+    C : float
+        Equivalent Celsius temperature.
 
     Notes
     -----
@@ -2543,12 +2623,10 @@ def F2C(F):
 
     Examples
     --------
-    >>> from fluids.core import F2C
-    >>> F2C(np.array([-40, 40.0]))
-    array([-40.        ,   4.44444444])
-
+    >>> F2C(-40.0)
+    -40.0
     """
-    return (np.asanyarray(F) - 32) / 1.8
+    return (F - 32.0) / 1.8
 
 
 def C2F(C):
@@ -2557,13 +2635,13 @@ def C2F(C):
 
     Parameters
     ----------
-    C : array_like
-        Celsius temperature(s) to be converted.
+    C : float
+        Celsius temperature to be converted.
 
     Returns
     -------
-    F : float or array of floats
-        Equivalent Fahrenheit temperature(s).
+    F : float
+        Equivalent Fahrenheit temperature.
 
     Notes
     -----
@@ -2571,12 +2649,10 @@ def C2F(C):
 
     Examples
     --------
-    >>> from fluids.core import C2F
-    >>> C2F(np.array([-40, 40.0]))
-    array([ -40.,  104.])
-
+    >>> C2F(-40.0)
+    -40.0
     """
-    return 1.8 * np.asanyarray(C) + 32
+    return 1.8*C + 32.0
 
 
 def F2K(F):
@@ -2585,13 +2661,13 @@ def F2K(F):
 
     Parameters
     ----------
-    F : array_like
-        Fahrenheit temperature(s) to be converted.
+    F : float
+        Fahrenheit temperature to be converted.
 
     Returns
     -------
-    K : float or array of floats
-        Equivalent Kelvin temperature(s).
+    K : float
+        Equivalent Kelvin temperature.
 
     Notes
     -----
@@ -2601,12 +2677,11 @@ def F2K(F):
 
     Examples
     --------
-    >>> from fluids.core import F2K
-    >>> F2K(np.array([-40, 104]))
-    array([ 233.15,  313.15])
+    >>> F2K(-40)
+    233.14999999999998
 
     """
-    return C2K(F2C(np.asanyarray(F)))
+    return (F - 32.0)/1.8 + zero_Celsius
 
 
 def K2F(K):
@@ -2615,13 +2690,13 @@ def K2F(K):
 
     Parameters
     ----------
-    K : array_like
-        Kelvin temperature(s) to be converted.
+    K : float
+        Kelvin temperature to be converted.
 
     Returns
     -------
-    F : float or array of floats
-        Equivalent Fahrenheit temperature(s).
+    F : float
+        Equivalent Fahrenheit temperature.
 
     Notes
     -----
@@ -2631,9 +2706,220 @@ def K2F(K):
 
     Examples
     --------
-    >>> from fluids.core import K2F
-    >>> K2F(np.array([233.15,  313.15]))
-    array([ -40.,  104.])
+    >>> K2F(233.15)
+    -39.99999999999996
 
     """
-    return C2F(K2C(np.asanyarray(K)))
+    return 1.8*(K - zero_Celsius) + 32.0
+
+
+def C2R(C):
+    """
+    Convert Celsius to Rankine
+
+    Parameters
+    ----------
+    C : float
+        Celsius temperature to be converted.
+
+    Returns
+    -------
+    Ra : float
+        Equivalent Rankine temperature.
+
+    Notes
+    -----
+    Computes ``Ra = 1.8 * (C + zero_Celsius)`` where `zero_Celsius` = 273.15,
+    i.e., (the absolute value of) temperature "absolute zero" as measured in
+    Celsius.
+
+    Examples
+    --------
+    >>> C2R(-40)
+    419.66999999999996
+    """
+    return 1.8 * (C + zero_Celsius)
+
+
+def K2R(K):
+    """
+    Convert Kelvin to Rankine
+
+    Parameters
+    ----------
+    K : float
+        Kelvin temperature to be converted.
+
+    Returns
+    -------
+    Ra : float
+        Equivalent Rankine temperature.
+
+    Notes
+    -----
+    Computes ``Ra = 1.8 * K``.
+
+    Examples
+    --------
+    >>> K2R(273.15)
+    491.66999999999996
+    """
+    return 1.8 * K
+
+
+def F2R(F):
+    """
+    Convert Fahrenheit to Rankine
+
+    Parameters
+    ----------
+    F : float
+        Fahrenheit temperature to be converted.
+
+    Returns
+    -------
+    Ra : float
+        Equivalent Rankine temperature.
+
+    Notes
+    -----
+    Computes ``Ra = F - 32 + 1.8 * zero_Celsius`` where `zero_Celsius` = 273.15, 
+    i.e., (the absolute value of) temperature "absolute zero" as measured in 
+    Celsius.
+
+    Examples
+    --------
+    >>> F2R(100)
+    559.67
+
+    """
+    return F - 32.0 + 1.8 * zero_Celsius
+
+
+def R2C(Ra):
+    """
+    Convert Rankine to Celsius
+
+    Parameters
+    ----------
+    Ra : float
+        Rankine temperature to be converted.
+
+    Returns
+    -------
+    C : float
+        Equivalent Celsius temperature.
+
+    Notes
+    -----
+    Computes ``C = Ra / 1.8 - zero_Celsius`` where `zero_Celsius` = 273.15, 
+    i.e., (the absolute value of) temperature "absolute zero" as measured in 
+    Celsius.
+
+    Examples
+    --------
+    >>> R2C(459.67)
+    -17.777777777777743
+
+    """
+    return Ra / 1.8 - zero_Celsius
+
+
+def R2K(Ra):
+    """
+    Convert Rankine to Kelvin
+
+    Parameters
+    ----------
+    Ra : float
+        Rankine temperature to be converted.
+
+    Returns
+    -------
+    K : float
+        Equivalent Kelvin temperature.
+
+    Notes
+    -----
+    Computes ``K = Ra / 1.8``.
+
+    Examples
+    --------
+    >>> R2K(491.67)
+    273.15
+    """
+    return Ra / 1.8
+    
+
+def R2F(Ra):
+    """
+    Convert Rankine to Fahrenheit
+
+    Parameters
+    ----------
+    Ra : float
+        Rankine temperature to be converted.
+
+    Returns
+    -------
+    F : float
+        Equivalent Fahrenheit temperature.
+
+    Notes
+    -----
+    Computes ``F = Ra + 32 - 1.8 * zero_Celsius`` where `zero_Celsius` = 273.15, 
+    i.e., (the absolute value of) temperature "absolute zero" as measured in 
+    Celsius.
+
+    Examples
+    --------
+    >>> R2F(491.67)
+    32.00000000000006
+    """
+    return Ra - 1.8*zero_Celsius + 32.0
+
+
+def Engauge_2d_parser(lines, flat=False):
+    '''Not exposed function to read a 2D file generated by engauge-digitizer;
+    for curve fitting.
+    '''
+    z_values = []
+    x_lists = []
+    y_lists = []
+    working_xs = []
+    working_ys = []
+    
+    new_curve = True
+    for line in lines:
+        if line.strip() == '':
+            new_curve = True
+        elif new_curve:
+            z = float(line.split(',')[1])
+            z_values.append(z)
+            if working_xs and working_ys:
+                x_lists.append(working_xs)
+                y_lists.append(working_ys)
+            working_xs = []
+            working_ys = []
+            new_curve = False
+        else:
+            x, y = [float(i) for i in line.strip().split(',')]
+            working_xs.append(x)
+            working_ys.append(y)
+    x_lists.append(working_xs)
+    y_lists.append(working_ys)
+    
+    if flat:
+        all_zs = []
+        all_xs = []
+        all_ys = []
+        for z, xs, ys in zip(z_values, x_lists, y_lists):
+            for x, y in zip(xs, ys):
+                all_zs.append(z)
+                all_xs.append(x)
+                all_ys.append(y)
+        return all_zs, all_xs, all_ys
+
+    return z_values, x_lists, y_lists
+
+
