@@ -22,18 +22,18 @@ SOFTWARE.'''
 
 from __future__ import division
 from math import exp, log, log10, tanh
-import numpy as np
-from scipy.constants import g
-from scipy.optimize import newton
-from scipy.integrate import odeint, cumtrapz
+from fluids.constants import g
+from fluids.numerics import secant
 from fluids.core import Reynolds
 
-__all__ = ['drag_sphere', 'v_terminal', 'integrate_drag_sphere', 'Stokes',
+__all__ = ['drag_sphere', 'drag_sphere_methods', 'v_terminal', 'integrate_drag_sphere',
+'time_v_terminal_Stokes', 'Stokes',
 'Barati', 'Barati_high', 'Rouse', 'Engelund_Hansen',
 'Clift_Gauvin', 'Morsi_Alexander', 'Graf', 'Flemmer_Banks', 'Khan_Richardson',
 'Swamee_Ojha', 'Yen', 'Haider_Levenspiel', 'Cheng', 'Terfous',
 'Mikhailov_Freire', 'Clift', 'Ceylan', 'Almedeij', 'Morrison', 'Song_Xu']
 
+__numba_additional_funcs__ = ['_v_terminal_err']
 
 def Stokes(Re):
     r'''Calculates drag coefficient of a smooth sphere using Stoke's law.
@@ -94,7 +94,7 @@ def Barati(Re):
 
     Examples
     --------
-    Maching example in [1]_, in a table of calculated values.
+    Matching example in [1]_, in a table of calculated values.
 
     >>> Barati(200.)
     0.7682237950389874
@@ -144,7 +144,7 @@ def Barati_high(Re):
 
     Examples
     --------
-    Maching example in [1]_, in a table of calculated values.
+    Matching example in [1]_, in a table of calculated values.
 
     >>> Barati_high(200.)
     0.7730544082789523
@@ -157,11 +157,17 @@ def Barati_high(Re):
        Evolutionary Approach." Powder Technology 257 (May 2014): 11-19.
        doi:10.1016/j.powtec.2014.02.045.
     '''
-    Cd = (8E-6*((Re/6530.)**2 + tanh(Re) - 8*log(Re)/log(10.))
-    - 0.4119*exp(-2.08E43/(Re+Re**2)**4)
-    - 2.1344*exp(-((log(Re**2 + 10.7563)/log(10))**2 + 9.9867)/Re)
-    + 0.1357*exp(-((Re/1620.)**2 + 10370.)/Re)
-    - 8.5E-3*(2*log(tanh(tanh(Re)))/log(10) - 2825.7162)/Re + 2.4795)
+    Re2 = Re*Re
+    t0 = 1.0/Re
+    t1 = (Re/6530.)
+    t2 = (Re/1620.)
+    t3 = log10(Re2 + 10.7563)
+    tanhRe = tanh(Re)
+    Cd = (8E-6*(t1*t1 + tanhRe - 8.0*log10(Re))
+    - 0.4119*exp(-2.08E43/(Re+Re2)**4)
+    - 2.1344*exp(-t0*(t3*t3 + 9.9867))
+    + 0.1357*exp(-t0*(t2*t2 + 10370.))
+    - 8.5E-3*t0*(2.0*log10(tanh(tanhRe)) - 2825.7162) + 2.4795)
     return Cd
 
 
@@ -399,6 +405,7 @@ def Flemmer_Banks(Re):
     .. math::
         C_D = \frac{24}{Re}10^E
 
+    .. math::
         E = 0.383Re^{0.356}-0.207Re^{0.396} - \frac{0.143}{1+(\log_{10} Re)^2}
 
     Parameters
@@ -853,12 +860,16 @@ def Almedeij(Re):
     .. math::
         C_D = \left[\frac{1}{(\phi_1 + \phi_2)^{-1} + (\phi_3)^{-1}} + \phi_4\right]^{0.1}
 
+    .. math::
         \phi_1 = (24Re^{-1})^{10} + (21Re^{-0.67})^{10} + (4Re^{-0.33})^{10} + 0.4^{10}
 
+    .. math::
         \phi_2 = \left[(0.148Re^{0.11})^{-10} + (0.5)^{-10}\right]^{-1}
 
+    .. math::
         \phi_3 = (1.57\times10^8Re^{-1.625})^{10}
 
+    .. math::
         \phi_4 = \left[(6\times10^{-17}Re^{2.63})^{-10} + (0.2)^{-10}\right]^{-1}
 
     Parameters
@@ -1013,6 +1024,40 @@ drag_sphere_correlations = {
     'Song_Xu': (Song_Xu, None, 1E3)
 }
 
+def drag_sphere_methods(Re, check_ranges=True):
+    r'''This function returns a list of methods that can be used to calculate
+    the drag coefficient of a sphere.
+    Twenty one methods are available, all requiring only the Reynolds number of 
+    the sphere. Most methods are valid from Re=0 to Re=200,000. 
+
+    Examples
+    --------
+    >>> len(drag_sphere_methods(200)
+    20
+    >>> len(drag_sphere_methods(200000, check_ranges=False))
+    21
+    >>> len(drag_sphere_methods(200000, check_ranges=True))
+    5
+
+    Parameters
+    ----------
+    Re : float
+        Particle Reynolds number of the sphere using the surrounding fluid
+        density and viscosity, [-]
+    check_ranges : bool, optional
+        Whether to return only correlations claiming to be valid for the given
+        `Re` or not, [-]
+    
+    Returns
+    -------
+    methods : list, only returned if AvailableMethods == True
+        List of methods which can be used to calculate `Cd` with the given `Re`
+    '''
+    methods = []
+    for key, (func, Re_min, Re_max) in drag_sphere_correlations.items():
+        if ((Re_min is None or Re > Re_min) and (Re_max is None or Re < Re_max)) or not check_ranges:
+            methods.append(key)
+    return methods
 
 def drag_sphere(Re, Method=None, AvailableMethods=False):
     r'''This function handles calculation of drag coefficient on spheres.
@@ -1057,17 +1102,13 @@ def drag_sphere(Re, Method=None, AvailableMethods=False):
         drag_sphere_correlations
     AvailableMethods : bool, optional
         If True, function will consider which methods which can be used to
-        calculate `Cd` with the given `Re`
+        calculate `Cd` with the given `Re`; DEPRECATED
     '''
-    def list_methods():
-        methods = []
-        for key, (func, Re_min, Re_max) in drag_sphere_correlations.items():
-            if (Re_min is None or Re > Re_min) and (Re_max is None or Re < Re_max):
-                methods.append(key)
-        return methods
     if AvailableMethods:
-        return list_methods()
-    if not Method:
+        import warnings
+        warnings.warn('Please use drag_sphere_methods', DeprecationWarning, stacklevel=2)
+        return drag_sphere_methods(Re)
+    if Method is None:
         if Re > 0.1:
             # Smooth transition point between the two models
             if Re <= 212963.26847812787:
@@ -1084,12 +1125,56 @@ def drag_sphere(Re, Method=None, AvailableMethods=False):
         else:
             return Stokes(Re)
 
-    if Method in drag_sphere_correlations:
-        return drag_sphere_correlations[Method][0](Re)
+    if Method == "Stokes":
+        return Stokes(Re)
+    elif Method == "Barati":
+        return Barati(Re)
+    elif Method == "Barati_high":
+        return Barati_high(Re)
+    elif Method == "Rouse":
+        return Rouse(Re)
+    elif Method == "Engelund_Hansen":
+        return Engelund_Hansen(Re)
+    elif Method == "Clift_Gauvin":
+        return Clift_Gauvin(Re)
+    elif Method == "Morsi_Alexander":
+        return Morsi_Alexander(Re)
+    elif Method == "Graf":
+        return Graf(Re)
+    elif Method == "Flemmer_Banks":
+        return Flemmer_Banks(Re)
+    elif Method == "Khan_Richardson":
+        return Khan_Richardson(Re)
+    elif Method == "Swamee_Ojha":
+        return Swamee_Ojha(Re)
+    elif Method == "Yen":
+        return Yen(Re)
+    elif Method == "Haider_Levenspiel":
+        return Haider_Levenspiel(Re)
+    elif Method == "Cheng":
+        return Cheng(Re)
+    elif Method == "Terfous":
+        return Terfous(Re)
+    elif Method == "Mikhailov_Freire":
+        return Mikhailov_Freire(Re)
+    elif Method == "Clift":
+        return Clift(Re)
+    elif Method == "Ceylan":
+        return Ceylan(Re)
+    elif Method == "Almedeij":
+        return Almedeij(Re)
+    elif Method == "Morrison":
+        return Morrison(Re)
+    elif Method == "Song_Xu":
+        return Song_Xu(Re)
     else:
-        raise Exception('Failure in in function')
+        raise ValueError('Unrecognized method')
 
 
+def _v_terminal_err(V, Method, Re_almost, main):
+    Cd = drag_sphere(Re_almost*V, Method=Method)
+    return V - (main/Cd)**0.5
+
 def v_terminal(D, rhop, rho, mu, Method=None):
     r'''Calculates terminal velocity of a falling sphere using any drag
     coefficient method supported by `drag_sphere`. The laminar solution for
@@ -1166,32 +1251,84 @@ def v_terminal(D, rhop, rho, mu, Method=None):
     main = 4/3.*g*D*(rhop-rho)/rho
     V_max = 1E6/rho/D*mu  # where the correlation breaks down, Re=1E6
 
-    def err(V):
-        Cd = drag_sphere(Re_almost*V, Method=Method)
-        return V - (main/Cd)**0.5
     # Begin the solver with 1/100 th the velocity possible at the maximum
     # Reynolds number the correlation is good for
-    return float(newton(err, V_max/100, tol=1E-12))
+    return secant(_v_terminal_err, V_max/100, xtol=1E-12, args=(Method, Re_almost, main))
 
 
-def time_v_terminal_Stokes(D, rhop, rho, mu, V0=0, tol=1e-14):
-    '''
+def time_v_terminal_Stokes(D, rhop, rho, mu, V0, tol=1e-14):
+    r'''Calculates the time required for a particle in Stoke's regime only to
+    reach terminal velocity (approximately). An infinitely long period is
+    required theoretically, but with floating points, it is possible to 
+    calculate the time required to come within a specified `tol` of that 
+    terminal velocity.
+    
+    .. math::
+        t_{term} = -\frac{1}{18\mu}\ln \left(\frac{D^2g\rho - D^2 g \rho_p 
+        + 18\mu V_{term}}{D^2g\rho - D^2 g \rho_p + 18\mu V_0 } \right) D^2 
+        \rho_p
+    
+    Parameters
+    ----------
+    D : float
+        Diameter of the sphere, [m]
+    rhop : float
+        Particle density, [kg/m^3]
+    rho : float
+        Density of the surrounding fluid, [kg/m^3]
+    mu : float
+        Viscosity of the surrounding fluid [Pa*s]
+    V0 : float
+        Initial velocity of the particle, [m/s]
+    tol : float, optional
+        How closely to approach the terminal velocity - the target velocity is
+        the terminal velocity multiplied by 1 (+/-) this, depending on if the
+        particle is accelerating or decelerating, [-]
+
+    Returns
+    -------
+    t : float
+        Time for the particle to reach the terminal velocity to within the
+        specified or an achievable tolerance, [s]
+
+    Notes
+    -----
+    The symbolic solution was obtained via Wolfram Alpha.
+    
+    If a solution cannot be obtained due to floating point error at very high
+    tolerance, an exception is raised - but first, the tolerance is doubled,
+    up to fifty times in an attempt to obtain the highest possible precision
+    while sill giving an answer. If at any point the tolerance is larger than
+    1%, an exception is also raised.
+
+    Examples
+    --------
+    >>> time_v_terminal_Stokes(D=1e-7, rhop=2200., rho=1.2, mu=1.78E-5, V0=1) 
+    3.1880031137871528e-06
+    >>> time_v_terminal_Stokes(D=1e-2, rhop=2200., rho=1.2, mu=1.78E-5, V0=1, 
+    ... tol=1e-30)
+    24800.636391801996
     '''
+    if tol < 1e-17:
+        tol = 2e-17
     term = D*D*g*rho - D*D*g*rhop 
     denominator = term + 18.*mu*V0
+    v_term_base = g*D*D*(rhop-rho)/(18.*mu)
+
+    const = D*D*rhop/mu*-1.0/18.
     for i in range(50):
-        try: 
-            v_term = g*D*D*(rhop-rho)/(18.*mu)
-            if v_term < V0:
-                v_term *= (1.0 + tol)
+        try:
+            if v_term_base < V0:
+                v_term = v_term_base*(1.0 + tol)
             else:
-                v_term *= (1.0 - tol)
+                v_term = v_term_base*(1.0 - tol)
             numerator = term + 18.*mu*v_term
-            return -1/18.*log(numerator/denominator)*D*D*rhop/mu
-        except ValueError:
-            tol = tol*2
+            return log(numerator/denominator)*const
+        except:
+            tol = tol + tol
             if tol > 0.01:
-                raise Exception('Could not find a solution')
+                raise ValueError('Could not find a solution')
+    raise ValueError('Could not find a solution')
 
 
 def integrate_drag_sphere(D, rhop, rho, mu, t, V=0, Method=None,
@@ -1243,12 +1380,21 @@ def integrate_drag_sphere(D, rhop, rho, mu, t, V=0, Method=None,
     .. math::
         V(t) = \frac{\exp(-at) (V_0 a + b(\exp(at) - 1))}{a}
         
+    .. math::
         x(t) = \frac{\exp(-a t)\left[V_0 a(\exp(a t) - 1) + b\exp(a t)(a t-1) 
         + b\right]}{a^2}
     
+    .. math::
         a = \frac{18\mu_f}{D^2\rho_p}
     
+    .. math::
         b = \frac{g(\rho_p-\rho_f)}{\rho_p}
+        
+    The analytical solution will automatically be used if the initial and 
+    terminal velocity is show the particle's behavior to be laminar. Note
+    that this behavior requires that the terminal velocity of the particle be
+    solved for - this adds slight (1%) overhead for the cases where particles 
+    are not laminar.
 
     Examples
     --------
@@ -1262,7 +1408,13 @@ def integrate_drag_sphere(D, rhop, rho, mu, t, V=0, Method=None,
        Fall of a Ball with Linear or Quadratic Drag." American Journal of 
        Physics 67, no. 6 (June 1999): 538-46. https://doi.org/10.1119/1.19320.
     '''
-    if Method == 'Stokes':
+    # Delayed import of necessaray functions
+    from scipy.integrate import odeint, cumtrapz
+    import numpy as np
+    laminar_initial = Reynolds(V=V, rho=rho, D=D, mu=mu) < 0.01
+    v_laminar_end_assumed = v_terminal(D=D, rhop=rhop, rho=rho, mu=mu, Method=Method)
+    laminar_end = Reynolds(V=v_laminar_end_assumed, rho=rho, D=D, mu=mu) < 0.01
+    if Method == 'Stokes' or (laminar_initial and laminar_end and Method is None):
         try:
             t1 = 18.0*mu/(D*D*rhop)
             t2 = g*(rhop-rho)/rhop
@@ -1323,6 +1475,7 @@ def integrate_drag_sphere(D, rhop, rho, mu, t, V=0, Method=None,
     # For an accurate integration of the particle's distance traveled
     pts = 1000 if distance else 2
     ts = np.linspace(0, t, pts)
+    
 
     # Perform the integration
     Vs = odeint(dv_dt, [V], ts)
