@@ -24,7 +24,7 @@ from __future__ import division
 from math import cos, sin, tan, atan, pi, radians, exp, acos, log10
 from fluids.friction import friction_factor
 from fluids.core import Froude_densimetric
-from fluids.numerics import interp, secant, py_brenth as brenth, NotBoundedError, secant
+from fluids.numerics import interp, secant, brenth, NotBoundedError
 from fluids.constants import g, inch
 
 __all__ = ['C_Reader_Harris_Gallagher',
@@ -49,6 +49,10 @@ __all__ = ['C_Reader_Harris_Gallagher',
            'C_Miller_1996',
            ]
 
+
+__numba_additional_funcs__ = ['err_dp_meter_solver_m', 'err_dp_meter_solver_P2',
+                              'err_dp_meter_solver_D2', 'err_dp_meter_solver_P1']
+
 CONCENTRIC_ORIFICE = 'orifice' # normal
 ECCENTRIC_ORIFICE = 'eccentric orifice'
 CONICAL_ORIFICE = 'conical orifice'
@@ -467,6 +471,19 @@ def C_Reader_Harris_Gallagher(D, Do, rho, mu, m, taps='corner'):
     return C
 
 
+_Miller_1996_unsupported_type = "Supported orifice types are %s" %str(
+        (CONCENTRIC_ORIFICE, SEGMENTAL_ORIFICE, ECCENTRIC_ORIFICE,
+         CONICAL_ORIFICE, QUARTER_CIRCLE_ORIFICE))
+_Miller_1996_unsupported_tap_concentric = "Supported taps for subtype '%s' are %s" %(
+        CONCENTRIC_ORIFICE, (ORIFICE_CORNER_TAPS, ORIFICE_FLANGE_TAPS, 
+                             ORIFICE_D_AND_D_2_TAPS, ORIFICE_PIPE_TAPS))
+_Miller_1996_unsupported_tap_pos_eccentric = "Supported tap positions for subtype '%s' are %s" %(
+        ECCENTRIC_ORIFICE, (TAPS_OPPOSITE, TAPS_SIDE))
+_Miller_1996_unsupported_tap_eccentric = "Supported taps for subtype '%s' are %s" %(
+        ECCENTRIC_ORIFICE, (ORIFICE_FLANGE_TAPS, ORIFICE_VENA_CONTRACTA_TAPS))
+_Miller_1996_unsupported_tap_segmental = "Supported taps for subtype '%s' are %s" %(
+        SEGMENTAL_ORIFICE, (ORIFICE_FLANGE_TAPS, ORIFICE_VENA_CONTRACTA_TAPS))
+
 def C_Miller_1996(D, Do, rho, mu, m, subtype='orifice',
                   taps=ORIFICE_CORNER_TAPS, tap_position=TAPS_OPPOSITE):
     r'''Calculates the coefficient of discharge of any of the orifice types 
@@ -702,12 +719,10 @@ def C_Miller_1996(D, Do, rho, mu, m, subtype='orifice',
         elif taps == ORIFICE_PIPE_TAPS:
             C_inf = 0.5959 + 0.461*beta**2.1 + 0.48*beta**8 + 0.039*beta**4/(1.0 - beta**4)
         else:
-            raise ValueError("Supported taps for subtype '%s' are %s" %(CONCENTRIC_ORIFICE, 
-                                (ORIFICE_CORNER_TAPS, ORIFICE_FLANGE_TAPS, ORIFICE_D_AND_D_2_TAPS, ORIFICE_PIPE_TAPS)))
+            raise ValueError(_Miller_1996_unsupported_tap_concentric)
     elif subtype == MILLER_ECCENTRIC_ORIFICE or subtype == ECCENTRIC_ORIFICE:
         if tap_position != TAPS_OPPOSITE and tap_position != TAPS_SIDE:
-            raise ValueError("Supported tap positions for subtype '%s' are %s" %(ECCENTRIC_ORIFICE, 
-                                (TAPS_OPPOSITE, TAPS_SIDE)))
+            raise ValueError(_Miller_1996_unsupported_tap_pos_eccentric)
         n = 0.75
         if taps == ORIFICE_FLANGE_TAPS:
             if tap_position == TAPS_OPPOSITE:
@@ -740,8 +755,7 @@ def C_Miller_1996(D, Do, rho, mu, m, subtype='orifice',
                     b = 52.8 - 434.2*beta + 1571.2*beta**2 - 2460.9*beta**3 + 1420.2*beta**4
                     C_inf = 0.5949 + 0.4078*beta**2.1 + 0.0547*beta**8 +0.0955*beta**4/(1-beta**4) - 0.5608*beta**3
         else:
-            raise ValueError("Supported taps for subtype '%s' are %s" %(ECCENTRIC_ORIFICE, 
-                            (ORIFICE_FLANGE_TAPS, ORIFICE_VENA_CONTRACTA_TAPS)))
+            raise ValueError(_Miller_1996_unsupported_tap_eccentric)
     elif subtype == MILLER_SEGMENTAL_ORIFICE or subtype == SEGMENTAL_ORIFICE:
         n = b = 0.0
         if taps == ORIFICE_FLANGE_TAPS:
@@ -756,8 +770,7 @@ def C_Miller_1996(D, Do, rho, mu, m, subtype='orifice',
                 # Yes these are supposed to be the same as the flange, large set
                 C_inf = 0.6276 + 0.0828*beta**2.1 + 0.2739*beta**8 - 0.0934*beta**4/(1-beta**4) - 0.1132*beta**3
         else:
-            raise ValueError("Supported taps for subtype '%s' are %s" %(SEGMENTAL_ORIFICE, 
-                            (ORIFICE_FLANGE_TAPS, ORIFICE_VENA_CONTRACTA_TAPS)))
+            raise ValueError(_Miller_1996_unsupported_tap_segmental)
     elif subtype == MILLER_CONICAL_ORIFICE or subtype == CONICAL_ORIFICE:
         n = b = 0.0
         if 250.0*beta <= Re <= 500.0*beta:
@@ -769,14 +782,9 @@ def C_Miller_1996(D, Do, rho, mu, m, subtype='orifice',
         C_inf = (0.7746 - 0.1334*beta**2.1 + 1.4098*beta**8 
                  + 0.0675*beta**4/(1.0 - beta**4) + 0.3865*beta**3)
     else:
-        raise ValueError("Supported orifice types are %s" %str((CONCENTRIC_ORIFICE, 
-                                                            SEGMENTAL_ORIFICE, 
-                                                            ECCENTRIC_ORIFICE,
-                                                            CONICAL_ORIFICE, 
-                                                            QUARTER_CIRCLE_ORIFICE)))
+        raise ValueError(_Miller_1996_unsupported_type)
     C = C_inf + b*Re**-n
     return C
-        
 
 def C_eccentric_orifice_ISO_15377_1998(D, Do):
     r'''Calculates the coefficient of discharge of an eccentric orifice based 
@@ -1214,16 +1222,17 @@ def nozzle_expansibility(D, Do, P1, P2, k, beta=None):
     tau = P2/P1
     term1 = k*tau**(2.0/k )/(k - 1.0)
     term2 = (1.0 - beta4)/(1.0 - beta4*tau**(2.0/k))
-    try:
-        term3 = (1.0 - tau**((k - 1.0)/k))/(1.0 - tau)
-    except ZeroDivisionError:
-        '''Obtained with:
+    if tau == 1.0:
+        '''Avoid a zero division error.
+        Obtained with:
             from sympy import *
             tau, k = symbols('tau, k')
             expr = (1 - tau**((k - 1)/k))/(1 - tau)
             limit(expr, tau, 1)
         '''
         term3 = (k - 1.0)/k
+    else:
+        term3 = (1.0 - tau**((k - 1.0)/k))/(1.0 - tau)
     return (term1*term2*term3)**0.5
 
 
@@ -2056,20 +2065,20 @@ CONE_METER_C = 0.82
 ISO_15377_CONICAL_ORIFICE_C = 0.734
 
 beta_simple_meters = frozenset([ISO_5167_ORIFICE, ISO_15377_ECCENTRIC_ORIFICE, 
-                          ISO_15377_CONICAL_ORIFICE, ISO_15377_QUARTER_CIRCLE_ORIFICE, 
-                          
-                          MILLER_ORIFICE, MILLER_ECCENTRIC_ORIFICE, 
-                          MILLER_SEGMENTAL_ORIFICE, MILLER_CONICAL_ORIFICE,
-                          MILLER_QUARTER_CIRCLE_ORIFICE, 
-                          
-                          CONCENTRIC_ORIFICE, ECCENTRIC_ORIFICE, CONICAL_ORIFICE,
-                          SEGMENTAL_ORIFICE, QUARTER_CIRCLE_ORIFICE,
-                          
-                          LONG_RADIUS_NOZZLE, 
-                          ISA_1932_NOZZLE, VENTURI_NOZZLE,
-                          AS_CAST_VENTURI_TUBE, 
-                          MACHINED_CONVERGENT_VENTURI_TUBE, 
-                          ROUGH_WELDED_CONVERGENT_VENTURI_TUBE])
+                      ISO_15377_CONICAL_ORIFICE, ISO_15377_QUARTER_CIRCLE_ORIFICE, 
+                      
+                      MILLER_ORIFICE, MILLER_ECCENTRIC_ORIFICE, 
+                      MILLER_SEGMENTAL_ORIFICE, MILLER_CONICAL_ORIFICE,
+                      MILLER_QUARTER_CIRCLE_ORIFICE, 
+                      
+                      CONCENTRIC_ORIFICE, ECCENTRIC_ORIFICE, CONICAL_ORIFICE,
+                      SEGMENTAL_ORIFICE, QUARTER_CIRCLE_ORIFICE,
+                      
+                      LONG_RADIUS_NOZZLE, 
+                      ISA_1932_NOZZLE, VENTURI_NOZZLE,
+                      AS_CAST_VENTURI_TUBE, 
+                      MACHINED_CONVERGENT_VENTURI_TUBE, 
+                      ROUGH_WELDED_CONVERGENT_VENTURI_TUBE])
 
 all_meters = set(beta_simple_meters)
 all_meters.update([CONE_METER, WEDGE_METER])
@@ -2206,11 +2215,21 @@ def differential_pressure_meter_C_epsilon(D, D2, m, P1, P2, rho, mu, k,
     ... meter_type='ISO 5167 orifice', taps='D')
     (0.6151252900244296, 0.9711026966676307)
     '''
-    # Translate default meter type to implementation specific correlation
-    try:
-        meter_type = _meter_type_to_corr_default[meter_type]
-    except KeyError:
-        pass
+#    # Translate default meter type to implementation specific correlation
+    if meter_type == CONCENTRIC_ORIFICE:
+        meter_type = ISO_5167_ORIFICE
+    elif meter_type == ECCENTRIC_ORIFICE:
+        meter_type = ISO_15377_ECCENTRIC_ORIFICE
+    elif meter_type == CONICAL_ORIFICE:
+        meter_type = ISO_15377_CONICAL_ORIFICE
+    elif meter_type == QUARTER_CIRCLE_ORIFICE:
+        meter_type = ISO_15377_QUARTER_CIRCLE_ORIFICE
+    elif meter_type == SEGMENTAL_ORIFICE:
+        meter_type = MILLER_SEGMENTAL_ORIFICE
+#    try:
+#        meter_type = _meter_type_to_corr_default[meter_type]
+#    except KeyError:
+#        pass
     
     if meter_type == ISO_5167_ORIFICE:
         C = C_Reader_Harris_Gallagher(D, D2, rho, mu, m, taps)
@@ -2269,6 +2288,40 @@ def differential_pressure_meter_C_epsilon(D, D2, m, P1, P2, rho, mu, k,
     return C, epsilon
 
 
+def err_dp_meter_solver_m(m_D, D, D2, P1, P2, rho, mu, k, meter_type, taps, tap_position):
+    m = m_D*D
+    C, epsilon = differential_pressure_meter_C_epsilon(D, D2, m, P1, P2, rho, 
+                                                  mu, k, meter_type, 
+                                                  taps=taps, tap_position=tap_position)
+    m_calc = flow_meter_discharge(D=D, Do=D2, P1=P1, P2=P2, rho=rho, 
+                                C=C, expansibility=epsilon)
+    err =  m - m_calc
+    return err
+
+def err_dp_meter_solver_P2(P2, D, D2, m, P1, rho, mu, k, meter_type, taps, tap_position):
+    C, epsilon = differential_pressure_meter_C_epsilon(D, D2, m, P1, P2, rho,
+                                                  mu, k, meter_type, 
+                                                  taps=taps, tap_position=tap_position)
+    m_calc = flow_meter_discharge(D=D, Do=D2, P1=P1, P2=P2, rho=rho, 
+                                C=C, expansibility=epsilon)
+    return m - m_calc
+
+def err_dp_meter_solver_D2(D2, D, m, P1, P2, rho, mu, k, meter_type, taps, tap_position):
+    C, epsilon = differential_pressure_meter_C_epsilon(D, D2, m, P1, P2, rho, 
+                                                  mu, k, meter_type, 
+                                                  taps=taps, tap_position=tap_position)
+    m_calc = flow_meter_discharge(D=D, Do=D2, P1=P1, P2=P2, rho=rho, 
+                                C=C, expansibility=epsilon)
+    return m - m_calc
+
+def err_dp_meter_solver_P1(P1, D, D2, m, P2, rho, mu, k, meter_type, taps, tap_position):
+    C, epsilon = differential_pressure_meter_C_epsilon(D, D2, m, P1, P2, rho, 
+                                                  mu, k, meter_type, 
+                                                  taps=taps, tap_position=tap_position)
+    m_calc = flow_meter_discharge(D=D, Do=D2, P1=P1, P2=P2, rho=rho, 
+                                C=C, expansibility=epsilon)
+    return m - m_calc
+
 def differential_pressure_meter_solver(D, rho, mu, k, D2=None, P1=None, P2=None, 
                                        m=None, meter_type=ISO_5167_ORIFICE, 
                                        taps=None, tap_position=None):
@@ -2355,58 +2408,30 @@ def differential_pressure_meter_solver(D, rho, mu, k, D2=None, P1=None, P2=None,
     ... meter_type='ISO 5167 orifice', taps='D')
     0.04999999990831885
     '''
-    if m is None and None not in (D, D2, P1, P2):
-        
-        def to_solve(m_D):
-            m = m_D*D
-            C, epsilon = differential_pressure_meter_C_epsilon(D, D2, m, P1, P2, rho, 
-                                                          mu, k, meter_type, 
-                                                          taps=taps, tap_position=tap_position)
-            m_calc = flow_meter_discharge(D=D, Do=D2, P1=P1, P2=P2, rho=rho, 
-                                        C=C, expansibility=epsilon)
-            err =  m - m_calc
-            return err
+    if m is None and D is not None and D2 is not None and P1 is not None and P2 is not None:
         # Diameter to mass flow ratio
         m_D_guess = 40
         if rho < 100.0:
             m_D_guess *= 1e-2
-        return secant(to_solve, m_D_guess)*D
-    elif D2 is None and None not in (D, m, P1, P2):
-        def to_solve(D2):
-            C, epsilon = differential_pressure_meter_C_epsilon(D, D2, m, P1, P2, rho, 
-                                                          mu, k, meter_type, 
-                                                          taps=taps, tap_position=tap_position)
-            m_calc = flow_meter_discharge(D=D, Do=D2, P1=P1, P2=P2, rho=rho, 
-                                        C=C, expansibility=epsilon)
-            return m - m_calc
+        return secant(err_dp_meter_solver_m, m_D_guess, args=(D, D2, P1, P2, rho, mu, k, meter_type, taps, tap_position))*D
+    elif D2 is None and D is not None and m is not None and P1 is not None and P2 is not None:
+        args = (D, m, P1, P2, rho, mu, k, meter_type, taps, tap_position)
         try:
-            return brenth(to_solve, D*(1-1E-9), D*5E-3)
-        except NotBoundedError:
-            return secant(to_solve, D*.3, high=D, low=D*1e-10)
-    elif P2 is None and None not in (D, D2, m, P1):
-        def to_solve(P2):
-            C, epsilon = differential_pressure_meter_C_epsilon(D, D2, m, P1, P2, rho,
-                                                          mu, k, meter_type, 
-                                                          taps=taps, tap_position=tap_position)
-            m_calc = flow_meter_discharge(D=D, Do=D2, P1=P1, P2=P2, rho=rho, 
-                                        C=C, expansibility=epsilon)
-            return m - m_calc
+            return brenth(err_dp_meter_solver_D2, D*(1-1E-9), D*5E-3, args=args)
+        except:
+            return secant(err_dp_meter_solver_D2, D*.3, args=args, high=D, low=D*1e-10)
+    elif P2 is None and D is not None and D2 is not None and m is not None and P1 is not None:
+        args = (D, D2, m, P1, rho, mu, k, meter_type, taps, tap_position)
         try:
-            return brenth(to_solve, P1*(1-1E-9), P1*0.5)
-        except NotBoundedError:
-            return secant(to_solve, P1*0.5, low=P1*1e-10, high=P1, bisection=True)
-    elif P1 is None and None not in (D, D2, m, P2):
-        def to_solve(P1):
-            C, epsilon = differential_pressure_meter_C_epsilon(D, D2, m, P1, P2, rho, 
-                                                          mu, k, meter_type, 
-                                                          taps=taps, tap_position=tap_position)
-            m_calc = flow_meter_discharge(D=D, Do=D2, P1=P1, P2=P2, rho=rho, 
-                                        C=C, expansibility=epsilon)
-            return m - m_calc
+            return brenth(err_dp_meter_solver_P2, P1*(1-1E-9), P1*0.5, args=args)
+        except:
+            return secant(err_dp_meter_solver_P2, P1*0.5, low=P1*1e-10, args=args, high=P1, bisection=True)
+    elif P1 is None and D is not None and D2 is not None and m is not None and P2 is not None:
+        args = (D, D2, m, P2, rho, mu, k, meter_type, taps, tap_position)
         try:
-            return brenth(to_solve, P2*(1+1E-9), P2*1.4)
-        except NotBoundedError:
-            return secant(to_solve, P2*1.5, low=P2, bisection=True)
+            return brenth(err_dp_meter_solver_P1, P2*(1+1E-9), P2*1.4, args=args)
+        except:
+            return secant(err_dp_meter_solver_P1, P2*1.5, args=args, low=P2, bisection=True)
     else:
         raise ValueError('Solver is capable of solving for one of P2, D2, or m only.')
 
@@ -2421,6 +2446,8 @@ _dP_orifice_set = set([ISO_5167_ORIFICE, ISO_15377_ECCENTRIC_ORIFICE,
                   CONCENTRIC_ORIFICE, ECCENTRIC_ORIFICE, CONICAL_ORIFICE,
                   SEGMENTAL_ORIFICE, QUARTER_CIRCLE_ORIFICE])
 
+_missing_C_msg = "Parameter C is required for this orifice type"
+
 def differential_pressure_meter_dP(D, D2, P1, P2, C=None, 
                                    meter_type=ISO_5167_ORIFICE):
     r'''Calculates either the non-recoverable pressure drop of a differential
@@ -2471,13 +2498,16 @@ def differential_pressure_meter_dP(D, D2, P1, P2, C=None,
     1788.5717754177406
     '''
     if meter_type in _dP_orifice_set:
+        if C is None: raise ValueError(_missing_C_msg)
         dP = dP_orifice(D=D, Do=D2, P1=P1, P2=P2, C=C)
     elif meter_type == LONG_RADIUS_NOZZLE:
+        if C is None: raise ValueError(_missing_C_msg)
         dP = dP_orifice(D=D, Do=D2, P1=P1, P2=P2, C=C)
     elif meter_type == ISA_1932_NOZZLE:
+        if C is None: raise ValueError(_missing_C_msg)
         dP = dP_orifice(D=D, Do=D2, P1=P1, P2=P2, C=C)
     elif meter_type == VENTURI_NOZZLE:
-        raise Exception(NotImplemented)
+        raise NotImplementedError("Venturi meter does not have an implemented pressure drop correlation")
     
     elif meter_type == AS_CAST_VENTURI_TUBE:
         dP = dP_venturi_tube(D=D, Do=D2, P1=P1, P2=P2)
