@@ -60,7 +60,7 @@ Demister Geometry
 from __future__ import division
 from math import log
 from fluids.constants import g, pi
-from fluids.numerics import newton, newton_system
+from fluids.numerics import secant, newton_system
 
 __all__ = ['voidage_experimental', 'specific_area_mesh',
 'Stichlmair_dry', 'Stichlmair_wet', 'Stichlmair_flood', 'Robbins',
@@ -68,7 +68,8 @@ __all__ = ['voidage_experimental', 'specific_area_mesh',
 'dP_demister_dry_Setekleiv_Svendsen',
 'dP_demister_wet_ElDessouky', 'separation_demister_ElDessouky']
 
-__numba_additional_funcs__ = ['_Stichlmair_flood_f', '_Stichlmair_flood_f_and_jac']
+__numba_additional_funcs__ = ['_Stichlmair_flood_f', '_Stichlmair_flood_f_and_jac',
+                              '_Stichlmair_wet_err']
 ### Demister
 
 def dP_demister_dry_Setekleiv_Svendsen(S, voidage, vs, rho, mu, L=1):
@@ -458,6 +459,11 @@ def Stichlmair_dry(Vg, rhog, mug, voidage, specific_area, C1, C2, C3, H=1.):
     return 3/4.*f0*(1-voidage)/voidage**4.65*rhog*H/dp*Vg**2
 
 
+def _Stichlmair_wet_err(dP_irr, h0, c1, dP_dry, H, voidage, c):
+    hT = h0*(1.0 + 20.0*dP_irr*dP_irr*c1)
+    err = dP_dry/H*((1-voidage+hT)/(1.0 - voidage))**((2.0 + c)/3.)*(voidage/(voidage-hT))**4.65 -dP_irr/H
+    return err
+
 def Stichlmair_wet(Vg, Vl, rhog, rhol, mug, voidage, specific_area, C1, C2, C3, H=1):
     r'''Calculates dry pressure drop across a packed column, using the
     Stichlmair [1]_ correlation. Uses three regressed constants for each
@@ -538,7 +544,7 @@ def Stichlmair_wet(Vg, Vl, rhog, rhol, mug, voidage, specific_area, C1, C2, C3,
 
     >>> Stichlmair_wet(Vg=0.4, Vl = 5E-3, rhog=5., rhol=1200., mug=5E-5,
     ... voidage=0.68, specific_area=260., C1=32., C2=7., C3=1.)
-    539.8768237253518
+    539.876823725352
 
     References
     ----------
@@ -558,11 +564,10 @@ def Stichlmair_wet(Vg, Vl, rhog, rhol, mug, voidage, specific_area, C1, C2, C3,
     c = (-C1/Re - C2/(2*Re**0.5))/f0
     Frl = Vl**2*specific_area/(g*voidage**4.65)
     h0 = 0.555*Frl**(1/3.)
-    def to_zero(dP_irr):
-        hT = h0*(1.0 + 20.0*(dP_irr/H/rhol/g)**2)
-        err = dP_dry/H*((1-voidage+hT)/(1.0 - voidage))**((2.0 + c)/3.)*(voidage/(voidage-hT))**4.65 -dP_irr/H
-        return err
-    return float(newton(to_zero, dP_dry))
+    
+    c1 = 1.0/(H*rhol*g)
+    c1 *= c1
+    return secant(_Stichlmair_wet_err, dP_dry, args=(h0, c1, dP_dry, H, voidage, c))
 
 
 def _Stichlmair_flood_f(inputs, Vl, rhog, rhol, mug, voidage, specific_area,
@@ -646,7 +651,13 @@ def _Stichlmair_flood_f_and_jac(inputs, Vl, rhog, rhol, mug, voidage,
              -1.85*x16*x29*x40*x5/x26,
              3285600.0*x42*(-x30 + 4.0)*x38*x38- 91668240.0*x42*x36*x36 - 2.0*x32/(dP_irr*x6))
     
-    return [F1, F2], [[dF1_dVg, dF2_dVg], [dF1_dP_irr, dF2_dP_irr]]
+    err = [0.0]*2
+    err[0] = F1
+    err[1] = F2
+    
+    jac = [[dF1_dVg, dF2_dVg], [dF1_dP_irr, dF2_dP_irr]]# numba: delete
+#    jac = np.array([[dF1_dVg, dF2_dVg], [dF1_dP_irr, dF2_dP_irr]]) # numba: uncomment
+    return err, jac
 
 
     
@@ -732,7 +743,9 @@ def Stichlmair_flood(Vl, rhog, rhol, mug, voidage, specific_area, C1, C2, C3,
        Packed Columns." Gas Separation & Purification 3, no. 1 (March 1989):
        19-28. doi:10.1016/0950-4214(89)80016-7.
     '''
-    guess =  [Vl*100., 1000.0]
+    guess = [0.0]*2
+    guess[0] = Vl*100.0
+    guess[1] = 1000.0
     return newton_system(_Stichlmair_flood_f_and_jac, x0=guess, jac=True,
                          args=(Vl, rhog, rhol, mug, voidage, specific_area, C1, 
                          C2, C3, H), ytol=1e-11)[0][0]
