@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''Chemical Engineering Design Library (ChEDL). Utilities for process modeling.
-Copyright (C) 2016, 2017 Caleb Bell <Caleb.Andrew.Bell@gmail.com>
+Copyright (C) 2016, 2017, 2018 Caleb Bell <Caleb.Andrew.Bell@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -22,10 +22,9 @@ SOFTWARE.'''
 
 from __future__ import division
 from math import exp, log, pi, sin, cos, radians
-from scipy.constants import g
+from fluids.constants import g
 from fluids.core import Froude
 
-
 __all__ = ['Thom', 'Zivi', 'Smith', 'Fauske', 'Chisholm_voidage', 'Turner_Wallis',
            'homogeneous', 'Chisholm_Armand', 'Armand', 'Nishino_Yamazaki',
            'Guzhov', 'Kawahara', 'Baroczy', 'Tandon_Varma_Gupta', 'Harms',
@@ -35,7 +34,9 @@ __all__ = ['Thom', 'Zivi', 'Smith', 'Fauske', 'Chisholm_voidage', 'Turner_Wallis
            'Sun_Duffey_Peng', 'Xu_Fang_voidage', 'Woldesemayat_Ghajar',
            'Lockhart_Martinelli_Xtt', 'two_phase_voidage_experimental', 
            'density_two_phase', 'Beattie_Whalley', 'McAdams', 'Cicchitti',
-           'Lin_Kwok', 'Fourar_Bories', 'liquid_gas_voidage', 'gas_liquid_viscosity', 
+           'Lin_Kwok', 'Fourar_Bories','Duckler', 'liquid_gas_voidage',
+           'liquid_gas_voidage_methods', 'gas_liquid_viscosity', 
+           'gas_liquid_viscosity_methods',
            'two_phase_voidage_correlations', 'liquid_gas_viscosity_correlations']
 
 ### Models based on slip ratio
@@ -409,7 +410,7 @@ def homogeneous(x, rhol, rhog):
        Upward Inclined Pipes." International Journal of Multiphase Flow 33, 
        no. 4 (April 2007): 347-370. doi:10.1016/j.ijmultiphaseflow.2006.09.004.
     '''
-    return 1./(1. + (1-x)/x*(rhog/rhol))
+    return 1.0/(1.0 + (1.0 - x)/x*(rhog/rhol))
 
 
 def Chisholm_Armand(x, rhol, rhog):
@@ -707,6 +708,7 @@ def Lockhart_Martinelli_Xtt(x, rhol, rhog, mul, mug, pow_x=0.9, pow_rho=0.5,
         X_{tt} = \left(\frac{1-x}{x}\right)^{0.9} \left(\frac{\rho_g}{\rho_l}
         \right)^{0.5}\left(\frac{\mu_l}{\mu_g}\right)^{0.1}
         
+    .. math::
         X_{tt} = \left(\frac{1-x}{x}\right)^{(2-n)/2} \left(\frac{\rho_g}
         {\rho_l}\right)^{0.5}\left(\frac{\mu_l}{\mu_g}\right)^{n/2}
         
@@ -756,7 +758,7 @@ def Lockhart_Martinelli_Xtt(x, rhol, rhog, mul, mug, pow_x=0.9, pow_rho=0.5,
        Upward Inclined Pipes." International Journal of Multiphase Flow 33, 
        no. 4 (April 2007): 347-370. doi:10.1016/j.ijmultiphaseflow.2006.09.004.
     '''
-    if n:
+    if n is not None:
         pow_x = (2-n)/2.
         pow_mu = n/2.
     return ((1-x)/x)**pow_x * (rhog/rhol)**pow_rho * (mul/mug)**pow_mu
@@ -914,6 +916,7 @@ def Harms(x, rhol, rhog, mul, mug, m, D):
         \alpha = \left[1 - 10.06Re_l^{-0.875}(1.74 + 0.104Re_l^{0.5})^2
         \left(1.376 + \frac{7.242}{X_{tt}^{1.655}}\right)^{-0.5}\right]^2
         
+    .. math::
         Re_l = \frac{G_{tp}(1-x)D}{\mu_l}
 
     Parameters
@@ -1043,8 +1046,10 @@ def Graham(x, rhol, rhog, mul, mug, m, D, g=g):
     .. math::
         \alpha = 1 - \exp\{-1 - 0.3\ln(Ft) - 0.0328[\ln(Ft)]^2\}
         
+    .. math::
         Ft = \left[\frac{G_{tp}^2 x^3}{(1-x)\rho_g^2gD}\right]^{0.5}
         
+    .. math::
         \alpha = 0 \text{ for } F_t \le 0.01032
         
     Parameters
@@ -1111,6 +1116,7 @@ def Yashar(x, rhol, rhog, mul, mug, m, D, g=g):
     .. math::
         \alpha = \left[1 + \frac{1}{Ft} + X_{tt}\right]^{-0.321}
         
+    .. math::
         Ft = \left[\frac{G_{tp}^2 x^3}{(1-x)\rho_g^2gD}\right]^{0.5}
                 
     Parameters
@@ -1231,8 +1237,10 @@ def Kopte_Newell_Chato(x, rhol, rhog, mul, mug, m, D, g=g):
         \alpha = 1.045 - \exp\{-1 - 0.342\ln(Ft) - 0.0268[\ln(Ft)]^2 
         + 0.00597[\ln(Ft)]^3\}
         
+    .. math::
         Ft = \left[\frac{G_{tp}^2 x^3}{(1-x)\rho_g^2gD}\right]^{0.5}
         
+    .. math::
         \alpha = \alpha_h \text{ for } F_t \le 0.044
         
     Parameters
@@ -1297,8 +1305,10 @@ def Steiner(x, rhol, rhog, sigma, m, D, g=g):
         \alpha = \frac{x}{\rho_g}\left[C_0\left(\frac{x}{\rho_g} + \frac{1-x}
         {\rho_l}\right) +\frac{v_{gm}}{G} \right]^{-1}
         
+    .. math::
         v_{gm} = \frac{1.18(1-x)}{\rho_l^{0.5}}[g\sigma(\rho_l-\rho_g)]^{0.25}
 
+    .. math::
         C_0 = 1 + 0.12(1-x)
         
     Parameters
@@ -1359,8 +1369,10 @@ def Rouhani_1(x, rhol, rhog, sigma, m, D, g=g):
         \alpha = \frac{x}{\rho_g}\left[C_0\left(\frac{x}{\rho_g} + \frac{1-x}
         {\rho_l}\right) +\frac{v_{gm}}{G} \right]^{-1}
         
+    .. math::
         v_{gm} = \frac{1.18(1-x)}{\rho_l^{0.5}}[g\sigma(\rho_l-\rho_g)]^{0.25}
 
+    .. math::
         C_0 = 1 + 0.2(1-x)
         
     Parameters
@@ -1422,8 +1434,10 @@ def Rouhani_2(x, rhol, rhog, sigma, m, D, g=g):
         \alpha = \frac{x}{\rho_g}\left[C_0\left(\frac{x}{\rho_g} + \frac{1-x}
         {\rho_l}\right) +\frac{v_{gm}}{G} \right]^{-1}
         
+    .. math::
         v_{gm} = \frac{1.18(1-x)}{\rho_l^{0.5}}[g\sigma(\rho_l-\rho_g)]^{0.25}
 
+    .. math::
         C_0 = 1 + 0.2(1-x)(gD)^{0.25}\left(\frac{\rho_l}{G_{tp}}\right)^{0.5}
         
     Parameters
@@ -1485,8 +1499,10 @@ def Nicklin_Wilkes_Davidson(x, rhol, rhog, m, D, g=g):
         \alpha = \frac{x}{\rho_g}\left[C_0\left(\frac{x}{\rho_g} + \frac{1-x}
         {\rho_l}\right) +\frac{v_{gm}}{G} \right]^{-1}
         
+    .. math::
         v_{gm} = 0.35\sqrt{gD}
 
+    .. math::
         C_0 = 1.2
         
     Parameters
@@ -1543,8 +1559,10 @@ def Gregory_Scott(x, rhol, rhog):
         \alpha = \frac{x}{\rho_g}\left[C_0\left(\frac{x}{\rho_g} + \frac{1-x}
         {\rho_l}\right) +\frac{v_{gm}}{G} \right]^{-1}
         
+    .. math::
         v_{gm} = 0
 
+    .. math::
         C_0 = 1.19
         
     Parameters
@@ -1595,15 +1613,20 @@ def Dix(x, rhol, rhog, sigma, m, D, g=g):
         \alpha = \frac{x}{\rho_g}\left[C_0\left(\frac{x}{\rho_g} + \frac{1-x}
         {\rho_l}\right) +\frac{v_{gm}}{G} \right]^{-1}
         
+    .. math::
         v_{gm} = 2.9\left(g\sigma\frac{\rho_l-\rho_g}{\rho_l^2}\right)^{0.25}
 
+    .. math::
         C_0 = \frac{v_{sg}}{v_m}\left[1 + \left(\frac{v_{sl}}{v_{sg}}\right)
         ^{\left(\left(\frac{\rho_g}{\rho_l}\right)^{0.1}\right)}\right]
 
+    .. math::
         v_{gs} = \frac{mx}{\rho_g \frac{\pi}{4}D^2}
         
+    .. math::
         v_{ls} = \frac{m(1-x)}{\rho_l \frac{\pi}{4}D^2}
         
+    .. math::
         v_m = v_{gs} + v_{ls}
         
     Parameters
@@ -1666,8 +1689,10 @@ def Sun_Duffey_Peng(x, rhol, rhog, sigma, m, D, P, Pc, g=g):
         \alpha = \frac{x}{\rho_g}\left[C_0\left(\frac{x}{\rho_g} + \frac{1-x}
         {\rho_l}\right) +\frac{v_{gm}}{G} \right]^{-1}
         
+    .. math::
         v_{gm} = 1.41\left[\frac{g\sigma(\rho_l-\rho_g)}{\rho_l^2}\right]^{0.25}
         
+    .. math::
         C_0 = \left(0.82 + 0.18\frac{P}{P_c}\right)^{-1}
         
     Parameters
@@ -1719,7 +1744,8 @@ def Sun_Duffey_Peng(x, rhol, rhog, sigma, m, D, P, Pc, g=g):
        no. 4 (April 2007): 347-370. doi:10.1016/j.ijmultiphaseflow.2006.09.004.
     '''
     G = m/(pi/4*D**2)
-    C0 = (0.82 + 0.18*P/Pc)**-1
+    Pr = P/Pc if Pc is not None else 0.5
+    C0 = (0.82 + 0.18*Pr)**-1
     vgm = 1.41*(g*sigma*(rhol-rhog)/rhol**2)**0.25
     return x/rhog*(C0*(x/rhog + (1-x)/rhol) + vgm/G)**-1
 
@@ -1789,8 +1815,10 @@ def Woldesemayat_Ghajar(x, rhol, rhog, sigma, m, D, P, angle=0, g=g):
         + 2.9\left[\frac{gD\sigma(1+\cos\theta)(\rho_l-\rho_g)}
         {\rho_l^2}\right]^{0.25}(1.22 + 1.22\sin\theta)^{\frac{P}{P_{atm}}}}
         
+    .. math::
         v_{gs} = \frac{mx}{\rho_g \frac{\pi}{4}D^2}
         
+    .. math::
         v_{ls} = \frac{m(1-x)}{\rho_l \frac{\pi}{4}D^2}
         
         
@@ -1841,43 +1869,101 @@ def Woldesemayat_Ghajar(x, rhol, rhog, sigma, m, D, P, angle=0, g=g):
     vls = m*(1-x)/(rhol*pi/4*D**2)
     first = vgs*(1 + (vls/vgs)**((rhog/rhol)**0.1))
     second = 2.9*((g*D*sigma*(1 + cos(radians(angle)))*(rhol-rhog))/rhol**2)**0.25
+    if P is None: P = 101325.0
     third = (1.22 + 1.22*sin(radians(angle)))**(101325./P)
     return vgs/(first + second*third)
 
 # x, rhol, rhog 2ill be the minimum inputs
 
-two_phase_voidage_correlations = {'Thom' : (Thom, ['x', 'rhol', 'rhog', 'mul', 'mug']),
-'Zivi' : (Zivi, ['x', 'rhol', 'rhog']),
-'Smith' : (Smith, ['x', 'rhol', 'rhog']),
-'Fauske' : (Fauske, ['x', 'rhol', 'rhog']),
-'Chisholm_voidage' : (Chisholm_voidage, ['x', 'rhol', 'rhog']),
-'Turner Wallis' : (Turner_Wallis, ['x', 'rhol', 'rhog', 'mul', 'mug']),
-'homogeneous' : (homogeneous, ['x', 'rhol', 'rhog']),
-'Chisholm Armand' : (Chisholm_Armand, ['x', 'rhol', 'rhog']),
-'Armand' : (Armand, ['x', 'rhol', 'rhog']),
-'Nishino Yamazaki' : (Nishino_Yamazaki, ['x', 'rhol', 'rhog']),
-'Guzhov' : (Guzhov, ['x', 'rhol', 'rhog', 'm', 'D']),
-'Kawahara' : (Kawahara, ['x', 'rhol', 'rhog', 'D']),
-'Baroczy' : (Baroczy, ['x', 'rhol', 'rhog', 'mul', 'mug']),
-'Tandon Varma Gupta' : (Tandon_Varma_Gupta, ['x', 'rhol', 'rhog', 'mul', 'mug', 'm', 'D']),
-'Harms' : (Harms, ['x', 'rhol', 'rhog', 'mul', 'mug', 'm', 'D']),
-'Domanski Didion' : (Domanski_Didion, ['x', 'rhol', 'rhog', 'mul', 'mug']),
-'Graham' : (Graham, ['x', 'rhol', 'rhog', 'mul', 'mug', 'm', 'D', 'g']),
-'Yashar' : (Yashar, ['x', 'rhol', 'rhog', 'mul', 'mug', 'm', 'D', 'g']),
-'Huq_Loth' : (Huq_Loth, ['x', 'rhol', 'rhog']),
-'Kopte_Newell_Chato' : (Kopte_Newell_Chato, ['x', 'rhol', 'rhog', 'mul', 'mug', 'm', 'D', 'g']),
-'Steiner' : (Steiner, ['x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'g']),
-'Rouhani 1' : (Rouhani_1, ['x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'g']),
-'Rouhani 2' : (Rouhani_2, ['x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'g']),
-'Nicklin Wilkes Davidson' : (Nicklin_Wilkes_Davidson, ['x', 'rhol', 'rhog', 'm', 'D', 'g']),
-'Gregory_Scott' : (Gregory_Scott, ['x', 'rhol', 'rhog']),
-'Dix' : (Dix, ['x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'g']),
-'Sun Duffey Peng' : (Sun_Duffey_Peng, ['x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'P', 'Pc', 'g']),
-'Xu Fang voidage' : (Xu_Fang_voidage, ['x', 'rhol', 'rhog', 'm', 'D', 'g']),
-'Woldesemayat Ghajar' : (Woldesemayat_Ghajar, ['x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'P', 'angle', 'g'])}
+two_phase_voidage_correlations = {'Thom' : (Thom, ('x', 'rhol', 'rhog', 'mul', 'mug')),
+'Zivi' : (Zivi, ('x', 'rhol', 'rhog')),
+'Smith' : (Smith, ('x', 'rhol', 'rhog')),
+'Fauske' : (Fauske, ('x', 'rhol', 'rhog')),
+'Chisholm_voidage' : (Chisholm_voidage, ('x', 'rhol', 'rhog')),
+'Turner Wallis' : (Turner_Wallis, ('x', 'rhol', 'rhog', 'mul', 'mug')),
+'homogeneous' : (homogeneous, ('x', 'rhol', 'rhog')),
+'Chisholm Armand' : (Chisholm_Armand, ('x', 'rhol', 'rhog')),
+'Armand' : (Armand, ('x', 'rhol', 'rhog')),
+'Nishino Yamazaki' : (Nishino_Yamazaki, ('x', 'rhol', 'rhog')),
+'Guzhov' : (Guzhov, ('x', 'rhol', 'rhog', 'm', 'D')),
+'Kawahara' : (Kawahara, ('x', 'rhol', 'rhog', 'D')),
+'Baroczy' : (Baroczy, ('x', 'rhol', 'rhog', 'mul', 'mug')),
+'Tandon Varma Gupta' : (Tandon_Varma_Gupta, ('x', 'rhol', 'rhog', 'mul', 'mug', 'm', 'D')),
+'Harms' : (Harms, ('x', 'rhol', 'rhog', 'mul', 'mug', 'm', 'D')),
+'Domanski Didion' : (Domanski_Didion, ('x', 'rhol', 'rhog', 'mul', 'mug')),
+'Graham' : (Graham, ('x', 'rhol', 'rhog', 'mul', 'mug', 'm', 'D', 'g')),
+'Yashar' : (Yashar, ('x', 'rhol', 'rhog', 'mul', 'mug', 'm', 'D', 'g')),
+'Huq_Loth' : (Huq_Loth, ('x', 'rhol', 'rhog')),
+'Kopte_Newell_Chato' : (Kopte_Newell_Chato, ('x', 'rhol', 'rhog', 'mul', 'mug', 'm', 'D', 'g')),
+'Steiner' : (Steiner, ('x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'g')),
+'Rouhani 1' : (Rouhani_1, ('x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'g')),
+'Rouhani 2' : (Rouhani_2, ('x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'g')),
+'Nicklin Wilkes Davidson' : (Nicklin_Wilkes_Davidson, ('x', 'rhol', 'rhog', 'm', 'D', 'g')),
+'Gregory_Scott' : (Gregory_Scott, ('x', 'rhol', 'rhog')),
+'Dix' : (Dix, ('x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'g')),
+'Sun Duffey Peng' : (Sun_Duffey_Peng, ('x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'P', 'Pc', 'g')),
+'Xu Fang voidage' : (Xu_Fang_voidage, ('x', 'rhol', 'rhog', 'm', 'D', 'g')),
+'Woldesemayat Ghajar' : (Woldesemayat_Ghajar, ('x', 'rhol', 'rhog', 'sigma', 'm', 'D', 'P', 'angle', 'g'))}
 
+_unknown_two_phase_voidage_corr = 'Method not recognized; available methods are %s' %list(two_phase_voidage_correlations.keys())
 # All the available arguments are: 
 #{'rhol', 'angle=0', 'x', 'P', 'mug', 'rhog', 'D', 'g', 'Pc', 'sigma', 'mul', 'm'}
+def liquid_gas_voidage_methods(x, rhol, rhog, D=None, m=None, mul=None, mug=None, 
+                               sigma=None, P=None, Pc=None, angle=0, g=g,
+                               check_ranges=False):
+    r'''This function returns a list of liquid-gas voidage correlation names
+    which can perform the calculation with the provided inputs. The holdup is
+    for two-phase liquid-gas flow inside channels. 29 calculation methods are 
+    available, with varying input requirements.
+
+    Parameters
+    ----------
+    x : float
+        Quality of fluid, [-]
+    rhol : float
+        Liquid density, [kg/m^3]
+    rhog : float
+        Gas density, [kg/m^3]
+    D : float, optional
+        Diameter of pipe, [m]
+    m : float, optional
+        Mass flow rate of fluid, [kg/s]
+    mul : float, optional
+        Viscosity of liquid, [Pa*s]
+    mug : float, optional
+        Viscosity of gas, [Pa*s]
+    sigma : float, optional
+        Surface tension, [N/m]
+    P : float, optional
+        Pressure of fluid, [Pa]
+    Pc : float, optional
+        Critical pressure of fluid, [Pa]
+    angle : float, optional
+        Angle of the channel with respect to the horizontal (vertical = 90), 
+        [degrees]
+    g : float, optional
+        Acceleration due to gravity, [m/s^2]
+    check_ranges : bool, optional
+        Added for future use only
+
+    Returns
+    -------
+    methods : list, only returned if AvailableMethods == True
+        List of methods which can be used to calculate two-phase liquid-gas 
+        voidage with the given inputs.
+
+    Examples
+    --------
+    >>> len(liquid_gas_voidage_methods(m=0.6, x=0.1, rhol=915., rhog=2.67, mul=180E-6, mug=14E-6, sigma=0.0487, D=0.05))
+    27
+    '''
+    vals = locals()
+    usable_methods = []
+    for method, value in two_phase_voidage_correlations.items():
+        f, args = value
+        if all(vals[i] is not None for i in args):
+            usable_methods.append(method)
+    return usable_methods
 
 def liquid_gas_voidage(x, rhol, rhog, D=None, m=None, mul=None, mug=None, 
                        sigma=None, P=None, Pc=None, angle=0, g=g, Method=None, 
@@ -1887,6 +1973,11 @@ def liquid_gas_voidage(x, rhol, rhog, D=None, m=None, mul=None, mug=None,
     varying input requirements. A correlation will be automatically selected if
     none is specified. The full list of correlation can be obtained with the 
     `AvailableMethods` flag.
+    
+    This function is used to calculate the (liquid) holdup as well, as:
+        
+    .. math::
+        \text{holdup} = 1 - \text{voidage}
 
     If no correlation is selected, the following rules are used, with the 
     earlier options attempted first:
@@ -1937,7 +2028,7 @@ def liquid_gas_voidage(x, rhol, rhog, D=None, m=None, mul=None, mug=None,
     AvailableMethods : bool, optional
         If True, function will consider which methods which can be used to
         calculate two-phase liquid-gas voidage with the given inputs and return
-        them as a list instead of performing a calculation.
+        them as a list instead of performing a calculation. DEPRECATED!
 
     Notes
     -----
@@ -1948,25 +2039,76 @@ def liquid_gas_voidage(x, rhol, rhog, D=None, m=None, mul=None, mug=None,
     ... sigma=0.0487, D=0.05)
     0.9744097632663492
     '''
-    def list_methods(myargs):
-        usable_methods = []
-        for method, value in two_phase_voidage_correlations.items():
-            f, args = value
-            if all(myargs[i] is not None for i in args):
-                usable_methods.append(method)
-        return usable_methods
     if AvailableMethods:
-        return list_methods(locals())
-    if not Method:
-        Method = 'homogeneous'
-    if Method in two_phase_voidage_correlations:
-        f, args = two_phase_voidage_correlations[Method]
-        kwargs = {}
-        for arg in args:
-            kwargs[arg] = locals()[arg]
-        return f(**kwargs)
+        import warnings
+        warnings.warn('Please use liquid_gas_voidage_methods', DeprecationWarning, stacklevel=2)
+        return liquid_gas_voidage_methods(x=x, rhol=rhol, rhog=rhog, D=D, m=m, mul=mul, mug=mug, 
+                                          sigma=sigma, P=P, Pc=Pc, angle=angle, g=g)
+    if Method is None:
+        Method2 = 'homogeneous'
     else:
-        raise Exception('Method not recognized; available methods are %s' %list(two_phase_voidage_correlations.keys()))
+        Method2 = Method
+        
+    if Method2 == "Thom":
+        return Thom(x=x, rhol=rhol, rhog=rhog, mul=mul, mug=mug)
+    elif Method2 == "Zivi":
+        return Zivi(x=x, rhol=rhol, rhog=rhog)
+    elif Method2 == "Smith":
+        return Smith(x=x, rhol=rhol, rhog=rhog)
+    elif Method2 == "Fauske":
+        return Fauske(x=x, rhol=rhol, rhog=rhog)
+    elif Method2 == "Chisholm_voidage":
+        return Chisholm_voidage(x=x, rhol=rhol, rhog=rhog)
+    elif Method2 == "Turner Wallis":
+        return Turner_Wallis(x=x, rhol=rhol, rhog=rhog, mul=mul, mug=mug)
+    elif Method2 == "homogeneous":
+        return homogeneous(x=x, rhol=rhol, rhog=rhog)
+    elif Method2 == "Chisholm Armand":
+        return Chisholm_Armand(x=x, rhol=rhol, rhog=rhog)
+    elif Method2 == "Armand":
+        return Armand(x=x, rhol=rhol, rhog=rhog)
+    elif Method2 == "Nishino Yamazaki":
+        return Nishino_Yamazaki(x=x, rhol=rhol, rhog=rhog)
+    elif Method2 == "Guzhov":
+        return Guzhov(x=x, rhol=rhol, rhog=rhog, m=m, D=D)
+    elif Method2 == "Kawahara":
+        return Kawahara(x=x, rhol=rhol, rhog=rhog, D=D)
+    elif Method2 == "Baroczy":
+        return Baroczy(x=x, rhol=rhol, rhog=rhog, mul=mul, mug=mug)
+    elif Method2 == "Tandon Varma Gupta":
+        return Tandon_Varma_Gupta(x=x, rhol=rhol, rhog=rhog, mul=mul, mug=mug, m=m, D=D)
+    elif Method2 == "Harms":
+        return Harms(x=x, rhol=rhol, rhog=rhog, mul=mul, mug=mug, m=m, D=D)
+    elif Method2 == "Domanski Didion":
+        return Domanski_Didion(x=x, rhol=rhol, rhog=rhog, mul=mul, mug=mug)
+    elif Method2 == "Graham":
+        return Graham(x=x, rhol=rhol, rhog=rhog, mul=mul, mug=mug, m=m, D=D, g=g)
+    elif Method2 == "Yashar":
+        return Yashar(x=x, rhol=rhol, rhog=rhog, mul=mul, mug=mug, m=m, D=D, g=g)
+    elif Method2 == "Huq_Loth":
+        return Huq_Loth(x=x, rhol=rhol, rhog=rhog)
+    elif Method2 == "Kopte_Newell_Chato":
+        return Kopte_Newell_Chato(x=x, rhol=rhol, rhog=rhog, mul=mul, mug=mug, m=m, D=D, g=g)
+    elif Method2 == "Steiner":
+        return Steiner(x=x, rhol=rhol, rhog=rhog, sigma=sigma, m=m, D=D, g=g)
+    elif Method2 == "Rouhani 1":
+        return Rouhani_1(x=x, rhol=rhol, rhog=rhog, sigma=sigma, m=m, D=D, g=g)
+    elif Method2 == "Rouhani 2":
+        return Rouhani_2(x=x, rhol=rhol, rhog=rhog, sigma=sigma, m=m, D=D, g=g)
+    elif Method2 == "Nicklin Wilkes Davidson":
+        return Nicklin_Wilkes_Davidson(x=x, rhol=rhol, rhog=rhog, m=m, D=D, g=g)
+    elif Method2 == "Gregory_Scott":
+        return Gregory_Scott(x=x, rhol=rhol, rhog=rhog)
+    elif Method2 == "Dix":
+        return Dix(x=x, rhol=rhol, rhog=rhog, sigma=sigma, m=m, D=D, g=g)
+    elif Method2 == "Sun Duffey Peng":
+        return Sun_Duffey_Peng(x=x, rhol=rhol, rhog=rhog, sigma=sigma, m=m, D=D, P=P, Pc=Pc, g=g)
+    elif Method2 == "Xu Fang voidage":
+        return Xu_Fang_voidage(x=x, rhol=rhol, rhog=rhog, m=m, D=D, g=g)
+    elif Method2 == "Woldesemayat Ghajar":
+        return Woldesemayat_Ghajar(x=x, rhol=rhol, rhog=rhog, sigma=sigma, m=m, D=D, P=P, angle=angle, g=g)
+    else:
+        raise ValueError(_unknown_two_phase_voidage_corr)
 
 
 def density_two_phase(alpha, rhol, rhog):
@@ -2067,6 +2209,7 @@ def Beattie_Whalley(x, mul, mug, rhol, rhog):
     .. math::
         \mu_m = \mu_l(1-\alpha_m)(1 + 2.5\alpha_m) + \mu_g\alpha_m
         
+    .. math::
         \alpha_m = \frac{1}{1 + \left(\frac{1-x}{x}\right)\frac{\rho_g}{\rho_l}}
         \text{(homogeneous model)}
 
@@ -2403,6 +2546,39 @@ liquid_gas_viscosity_correlations = {'Beattie Whalley': (Beattie_Whalley, 1),
                                      'Cicchitti': (Cicchitti, 0),
                                      'Lin Kwok': (Lin_Kwok, 0)}
 
+def gas_liquid_viscosity_methods(rhol=None, rhog=None, check_ranges=False):
+    r'''This function returns a list of methods which can be used for calculating
+    two-phase liquid-gas viscosity.
+    Six calculation methods are available; three of them require only `x`, 
+    `mul`, and `mug`; the other three require `rhol` and `rhog` as well. 
+        
+    Parameters
+    ----------
+    rhol : float, optional
+        Liquid density, [kg/m^3]
+    rhog : float, optional
+        Gas density, [kg/m^3]
+    check_ranges : bool, optional
+        Added for compatibility only, never used
+    
+    Returns
+    -------
+    methods : list
+        List of methods which can be used to calculate two-phase liquid-gas
+        viscosity with the given inputs.
+    
+    Examples
+    --------
+    >>> gas_liquid_viscosity_methods()
+    ['McAdams', 'Cicchitti', 'Lin Kwok']
+    >>> gas_liquid_viscosity_methods(rhol=1000, rhog=2)
+    ['Beattie Whalley', 'Fourar Bories', 'Duckler', 'McAdams', 'Cicchitti', 'Lin Kwok']
+    '''
+    methods = ['McAdams', 'Cicchitti', 'Lin Kwok']
+    if rhol is not None and rhog is not None:
+        methods = list(liquid_gas_viscosity_correlations.keys())
+    return methods
+_gas_liquid_viscosity_method_unknown = 'Method not recognized; available methods are %s' %list(liquid_gas_viscosity_correlations.keys())
 
 
 def gas_liquid_viscosity(x, mul, mug, rhol=None, rhog=None, Method=None, 
@@ -2416,7 +2592,7 @@ def gas_liquid_viscosity(x, mul, mug, rhol=None, rhog=None, Method=None,
     flag.
     
     **ALL OF THESE METHODS ARE ONLY SUGGESTED DEFINITIONS, POTENTIALLY 
-    USEFUL FOR EMPERICAL WORK ONLY!**
+    USEFUL FOR EMPIRICAL WORK ONLY!**
 
     Parameters
     ----------
@@ -2448,7 +2624,7 @@ def gas_liquid_viscosity(x, mul, mug, rhol=None, rhog=None, Method=None,
     AvailableMethods : bool, optional
         If True, function will consider which methods which can be used to
         calculate two-phase liquid-gas viscosity with the given inputs and 
-        return them as a list instead of performing a calculation.
+        return them as a list instead of performing a calculation. DEPRECATED!
 
     Notes
     -----
@@ -2465,21 +2641,24 @@ def gas_liquid_viscosity(x, mul, mug, rhol=None, rhog=None, Method=None,
     >>> gas_liquid_viscosity(x=0.4, mul=1E-3, mug=1E-5)
     2.4630541871921184e-05
     '''
-    def list_methods():
-        methods = ['McAdams', 'Cicchitti', 'Lin Kwok']
-        if rhol is not None and rhog is not None:
-            methods = list(liquid_gas_viscosity_correlations.keys())
-        return methods
     if AvailableMethods:
-        return list_methods()
-    if not Method:
+        import warnings
+        warnings.warn('Please use gas_liquid_viscosity_methods', DeprecationWarning, stacklevel=2)
+        return gas_liquid_viscosity_methods(rhol, rhog)
+    if Method is None:
         Method = 'McAdams'
-
-    if Method in liquid_gas_viscosity_correlations:
-        f, i = liquid_gas_viscosity_correlations[Method]
-        if i == 0:
-            return f(x, mul, mug)
-        elif i == 1:
-            return f(x, mul, mug, rhol=rhol, rhog=rhog)
+        
+    if Method == 'Beattie Whalley':
+        return Beattie_Whalley(x, mul, mug, rhol=rhol, rhog=rhog)
+    elif Method == 'Fourar Bories':
+        return Fourar_Bories(x, mul, mug, rhol=rhol, rhog=rhog)
+    elif Method == 'Duckler':
+        return Duckler(x, mul, mug, rhol=rhol, rhog=rhog)
+    elif Method == 'McAdams':
+        return McAdams(x, mul, mug)
+    elif Method == 'Cicchitti':
+        return Cicchitti(x, mul, mug)
+    elif Method == 'Lin Kwok':
+        return Lin_Kwok(x, mul, mug)
     else:
-        raise Exception('Method not recognized; available methods are %s' %list(liquid_gas_viscosity_correlations.keys()))
+        raise ValueError(_gas_liquid_viscosity_method_unknown)
